@@ -1,12 +1,18 @@
 /***
 |''Name''|ImageMacroPlugin|
-|''Version''|0.7.2|
+|''Version''|0.8.3|
 |''Description''|Allows the rendering of svg images in a TiddlyWiki|
 |''Author''|Osmosoft|
 |''License''|[[BSD|http://www.opensource.org/licenses/bsd-license.php]]|
 |''Notes''|Currently only works in modern browsers (not IE)|
 |''Requires''|BinaryTiddlersPlugin|
-|''Usage''|{{{<<image SVG>>}}} will render the text of the tiddler with title SVG as an SVG image (but not in ie where it will fail silently)|
+!Usage
+{{{<<image SVG>>}}} will render the text of the tiddler with title SVG as an SVG image (but not in ie where it will fail silently)
+!!Parameters
+width/height: specify width/height parameters
+link: make the image link to a given location
+tiddlyLink: link to a tiddler
+
 !Notes
 Binary tiddlers in TiddlyWeb when passed through the wikifier will be shown as images.
 eg. {{{<<view text wikified>>}}} on a binary tiddler will show the image.
@@ -17,8 +23,8 @@ will create an image tag where the tiddler has content type beginning image and
 will attempt to create svg object in other scenarios
 {{{<<image /photos/x.jpg>>}}}
 will create an image tag with src /photos/x.jpg as long as there is not a tiddler called /photos/x.jpg in 
-which case it will render that tiddler as an image.
-
+which case it will render that tiddler as an image. Note for the case of svg files it will attempt to render as an svg if possible via the image
+tag. It doesn't embed the svg in the dom for security reasons as svg code can contain javascript.
 !Code
 ***/
 //{{{
@@ -31,8 +37,12 @@ var macro = config.macros.image = {
 	_fixPrefix: 1,
 	_external_cache: {},
 	_image_tag_cache: {},
+	_image_dimensions: {},
 	refreshImage: function(src) {
 		var elements = macro._image_tag_cache[src] ? macro._image_tag_cache[src] : [];
+		if(macro._image_dimensions[src]) {
+			macro._image_dimensions[src] = false;
+		}
 		for(var i = 0; i < elements.length; i++) {
 			var el = $(elements[i]);
 			var newSrc = "%0?nocache=%1".format([src, Math.random()]);
@@ -102,6 +112,11 @@ var macro = config.macros.image = {
 		if(!options) {
 			options = {};
 		}
+		if(options.link) {
+			place = $("<a />").attr("href", options.link).appendTo(place)[0];
+		} else if(options.tiddlyLink) {
+			place = createTiddlyLink(place, options.tiddlyLink, false);
+		}
 		var tiddlerText = options.tiddler.text;
 		var svgDoc;
 		if (window.DOMParser) {
@@ -161,8 +176,11 @@ var macro = config.macros.image = {
 				place.appendChild(el);
 			}
 			// if a tiddler attribute is set this is read as a link
-			jQuery("[tiddler]", place).click(function(ev) {
-				var tiddler = $(ev.target).attr("tiddler");
+			$("[tiddler], [tiddlyLink]", place).attr("refresh", "link").click(function(ev) {
+				var tiddler = $(ev.target).attr("tiddlyLink");
+				if(!tiddler) { // backwards compatibility
+					tiddler = $(ev.target).attr("tiddler");
+				}
 				if(tiddler) {
 					story.displayTiddler(ev.target, tiddler);
 				}
@@ -176,7 +194,7 @@ var macro = config.macros.image = {
 	renderBinaryImageTiddler: function(place, tiddler, options) {
 		var resourceURI;
 		var fields = tiddler.fields;
-		if(fields["server.type"] == 'tiddlyweb') { // construct an accurate url for the resource  
+		if(fields["server.type"] == 'tiddlyweb') { // construct an accurate url for the resource	
 			resourceURI = "%0%1/tiddlers/%2".format([fields['server.host'],
 				fields['server.workspace'],fields['server.title']]);
 		} else { // guess the url for the resource
@@ -195,6 +213,7 @@ var macro = config.macros.image = {
 	},
 	renderImage: function(place, imageSource, options) {
 		var imageTiddler = store.getTiddler(imageSource);
+		options = options ? options : {};
 		if(imageTiddler && macro.isBinaryImageTiddler(imageTiddler)) { // handle the case where we have an image url
 			return macro.renderBinaryImageTiddler(place, imageTiddler, options);
 		} else if(imageTiddler){ // handle the case where we have a tiddler
@@ -202,58 +221,22 @@ var macro = config.macros.image = {
 		} else { // we have a string representing a url
 			// check if we can access the json format of this url
 			var newplace = $('<div class="externalImage"/>').appendTo(place)[0];
-			try{
-				var img = new Image();
-				img.onload = function(ev) { 
-					return macro.renderBinaryImageUrl(newplace, imageSource, options);
-				}; 
-				img.onerror = function(ev) {
-					var renderTiddler = function(tiddler, contentType) {
-						macro._external_cache[imageSource] = {tiddler: tiddler, contentType: contentType};
-						if(contentType == 'application/json') { // assume tiddlyweb server
-							contentType = tiddler.type;
-						} else { // try and use the response as tiddler text
-							tiddler = {text: tiddler};
-						}
-						if(macro.isBinaryImageType(contentType)) {
-							options.src = imageSource;
-							return macro.renderBinaryImageTiddler(newplace, tiddler, options);
-						} else {
-							return macro.renderSVGTiddler(newplace, tiddler, options);
-						}
+			if(!macro._image_tag_cache[imageSource]) {
+				try{
+					var img = new Image();
+					img.onload = function(ev) { 
+						return macro.renderBinaryImageUrl(newplace, imageSource, options);
+					}; 
+					img.onerror = function(ev) {
+						// image doesn't exist.
+						macro.renderAlternateText(newplace, options);
 					};
-
-					var cached = macro._external_cache[imageSource];
-					if(cached) { // use cache where possible
-						renderTiddler(cached.tiddler, cached.contentType)
-					} else {
-						ajaxReq({ // deal with tiddlyweb binary images
-							url: imageSource,
-							beforeSend: function(xhr) {
-								xhr.setRequestHeader("Accept", "application/json,*/*");
-								xhr.setRequestHeader("X-ControlView", "false"); // for tiddlyspace usage
-							},
-							success: function(tiddler, status, xhr) {
-								if(!tiddler) {
-									return macro.renderBinaryImageUrl(newplace, imageSource, options);
-								}
-								var header = xhr.getResponseHeader("content-type");
-								var contentType;
-								if(header) {
-									header = header ? header.split(";") : [];
-									contentType = header[0] || false;
-								}
-								renderTiddler(tiddler, contentType);
-							},
-							error: function() {
-								return macro.renderBinaryImageUrl(newplace, imageSource, options);
-							}
-						});
-					}
-				};
-				img.src = imageSource;
-			} catch(e) { // the url is external thus our ajax request failed. we could try proxying..
-				return macro.renderBinaryImageUrl(newplace, imageSource, options); // attempt to render as image
+					img.src = imageSource;
+				} catch(e) { // the url is external thus our ajax request failed. we could try proxying..
+					return macro.renderBinaryImageUrl(newplace, imageSource, options); // attempt to render as image
+				}
+			} else {
+				return macro.renderBinaryImageUrl(newplace, imageSource, options);
 			}
 		}
 	},
@@ -261,11 +244,20 @@ var macro = config.macros.image = {
 		var imageSource = params[0];
 		// collect named arguments
 		var args = macro.getArguments(paramString, params);
-		this.renderImage(place, imageSource, args);		
+		this.renderImage(place, imageSource, args);
 	},
 	renderAlternateText: function(place, options) {
 		if(options.alt) {
-			$("<div class='svgImageTest svgIconText' />").text(options.alt).appendTo(place);
+			var classNames = "svgImageText svgIconText";
+			if(options.link) {
+				$("<a />").addClass(classNames).attr("href", options.link).
+					text(options.alt).appendTo(place); 
+			} else if(options.tiddlyLink) {
+				var btn = createTiddlyLink(place, options.tiddlyLink, false, classNames);
+				$(btn).text(options.alt);
+			} else {
+				$("<span />").addClass(classNames).text(options.alt).appendTo(place); 
+			}
 		}
 	},
 	renderSVGTiddler: function(place, tiddler, options) {
@@ -274,7 +266,7 @@ var macro = config.macros.image = {
 		}
 		options.tiddler = tiddler;
 		options.fix = true;
-		
+
 		if(macro.svgAvailable) {
 			this.importSVG(place, options); // display the svg
 		} else {
@@ -283,13 +275,33 @@ var macro = config.macros.image = {
 		}
 	},
 	renderBinaryImageUrl: function(place, src, options) {
+		var srcUrl = options.src ? options.src : src;
 		var container = $('<div class="image" />').appendTo(place)[0];
+		if(options.link) {
+			container = $("<a />").attr("href", options.link).appendTo(container)[0];
+		} else if(options.tiddlyLink) {
+			container = createTiddlyLink(container, options.tiddlyLink, false);
+		}
+		var image_dimensions = macro._image_dimensions[srcUrl];
 		var image = new Image(); // due to weird scaling issues where you use just a width or just a height
-		var createImageTag = function(userW, userH) {
+		var createImageTag = function(dimensions) {
+			var userH = options.height;
+			var userW = options.width;
+			var w = dimensions ? dimensions.width : 0;
+			var h = dimensions ? dimensions.height : 0;
+			var ratio;
+			var preserveWidth = options.preserveAspectRatio && w > h;
+			var preserveHeight = options.preserveAspectRatio && h > w;
+			if(userH && !userW || preserveHeight) {
+				ratio = userH / h;
+				userW = ratio * w;
+			} else if (userW && !userH || preserveWidth) {
+				ratio = userW / w;
+				userH = ratio * h;
+			} 
 			var img = $("<img />");
 			img.attr("src", src);
 			img.appendTo(container);
-			var srcUrl = options.src ? options.src : src;
 			if(!macro._image_tag_cache[srcUrl]) {
 				macro._image_tag_cache[srcUrl] = [];
 			}
@@ -303,25 +315,19 @@ var macro = config.macros.image = {
 			img.addClass(options.imageClass);
 		};
 		
-		image.onload = function() {
-			var w = image.width;
-			var h = image.height;
-			var userH = options.height;
-			var userW = options.width;
-			var ratio;
-			if(userH && !userW) {
-				ratio = userH / h;
-				userW = ratio * w;
-			} else if (userW && !userH) {
-				ratio = userW / w;
-				userH = ratio * h;
-			}
-			createImageTag(userW, userH);
-		};
-		image.onerror = function() {
-			createImageTag(options.width, options.height);
-		};
-		image.src = src;
+		if(!image_dimensions) {
+			image.onload = function() {
+				var dimensions = { width: image.width, height: image.height};
+				macro._image_dimensions[srcUrl] = dimensions;
+				createImageTag(dimensions);
+			};
+			image.onerror = function() {
+				createImageTag(false);
+			};
+			image.src = src;
+		} else {
+			createImageTag(image_dimensions);
+		}
 	},
 	getArguments: function(paramString, params) {
 		var args = paramString.parseParams("name", null, true, false, true)[0];
@@ -346,6 +352,10 @@ var macro = config.macros.image = {
 		}
 		options.width = macro.lookupArgument(options, "width", width);
 		options.height = macro.lookupArgument(options, "height", height);
+		options.preserveAspectRatio = args.preserveAspectRatio && args.preserveAspectRatio[0] == "yes" 
+			? true : false;
+		options.tiddlyLink = macro.lookupArgument(options, "tiddlyLink", false);
+		options.link = macro.lookupArgument(options, "link", false);
 		return options;
 	},
 	lookupArgument: function(args, id, ifEmpty) {
@@ -392,7 +402,9 @@ config.macros.view.views.image = function(value, place, params, wikifier, paramS
 	invokeMacro(place, "image", "%0 %1".format([title, params.splice(2).join(" ")]), null, tiddler);
 };
 
-config.shadowTiddlers.StyleSheetImageMacro = ".wikifiedImage svg, .wikifiedImage .image { width: 80%; }";
+config.shadowTiddlers.StyleSheetImageMacro = [".wikifiedImage svg, .wikifiedImage .image { width: 80%; }",
+	".svgImageText {background-color:[[ColorPalette::Error]]; color:#ddd; padding: 5px; display: inline-block; }"
+].join("");
 store.addNotification("StyleSheetImageMacro", refreshStyles);
 
 })(jQuery);
